import requests

from config.settings import DING_ROBOT


class SendDingTalk:
    def __init__(self, robot):
        self.taskName = DING_ROBOT
        self.host = f"{DING_ROBOT[robot]['host']}?access_token={DING_ROBOT[robot]['token']}"

    def send_dingtalk_message(self, content):
        data = {'msgtype': 'text', 'text': {'content': '用户标签 ' + content}}
        response = None
        if isinstance(self.host, str):
            response = requests.post(self.host, json=data)
        else:
            for url in self.host:
                response = requests.post(url, json=data)
        return response


def send_dingtalk_message(message='', channel='default'):
    SendDingTalk(channel).send_dingtalk_message(message)


if __name__ == '__main__':
    SendDingTalk('default').send_dingtalk_message("你好")



import json
import logging
import requests
from proj.celery import app
from tgaflow.base import BaseTask

logger = logging.getLogger('task')


class RefreshTask(BaseTask):
    def __init__(self, flow_id, params):
        self.task_name = 'refresh'
        super().__init__(flow_id, params, [])
        self.url = self.settings['base_url'] + '/open/user-cluster-refresh'

    def process(self):
        try:
            result = requests.post(self.url, headers={
                'Content-Type': 'application/json',
            }, params={
                ('token', self.settings['token']),
                ('projectId', self.params['projectId']),
                ('clusterId', self.params['clusterId']),
            }, timeout=300)
            logger.info('refreshtask success: %s' % result.text)
            resultArr = json.loads(result.text)
            if 'return_code' in resultArr and resultArr['return_code'] != 0:
                return False, {'result': result.text}
            return True, {}
        except Exception as e:
            logger.info('refreshtask error: %s' % e)
            return False, {'result': str(e)}



import os.path
import time
from config.settings import DATA_BASE
from pybloom_live import ScalableBloomFilter


def ensure_path(path):
    if not os.path.exists(path):
        os.makedirs(path)


class BloomDeduplicate:
    # 转发表的ID
    def __init__(self, forward_id):
        self.forward_id = forward_id
        bloom_path = os.path.join(DATA_BASE, 'bloom')
        ensure_path(bloom_path)
        self.file_path = os.path.join(bloom_path, "forward_" + str(forward_id) + '.bloom')
        if not os.path.isfile(self.file_path):
            self.bloom = ScalableBloomFilter(initial_capacity=10000, error_rate=0.001)
        else:
            self.f = open(self.file_path, "rb")
            self.bloom = ScalableBloomFilter.fromfile(open(self.file_path, "rb"))
        self.wf = open(self.file_path, "wb")

    # True 代表已经转发过
    def is_exist(self, item):
        result = False
        if item in self.bloom:
            result = True
        else:
            self.bloom.add(item)
        return result

    def __del__(self):
        self.bloom.tofile(self.wf)


if __name__ == "__main__":
    bd = BloomDeduplicate("1")
    start = time.time()
    for i in range(0, 2000):
        if not bd.is_exist(10000 + i):
            print("error----")

        if i % 100000 == 0:
            print(i, time.time() - start, "||||||||")



from tgaflow.check import CheckTask
from tgaflow.download import DownloadTask
from tgaflow.refresh import RefreshTask
from proj.celery import app


# params={'projectId': 4, 'clusterId': 168}
@app.task
def refresh(flow_id, params):
    refreshTask = RefreshTask(flow_id, params)
    refreshTask.handle()


# params={'projectId': 4, 'clusterId': 168}
@app.task
def check(obj, flow_id, params):
    checkTask = CheckTask(flow_id, params)
    checkTask.handle()


# params={'name': 'test', 'sql': 'select * from'}
@app.task
def download(obj, flow_id, params):
    downloadTask = DownloadTask(flow_id, params)
    downloadTask.handle()


from tgaflow.base import BaseTask


class TransferBaseTask(BaseTask):
    def __init__(self, flow_id, params):
        self.task_name = 'transfer'
        super().__init__(flow_id, params, ['download'])

    def process(self):
        pass


import logging

from config.settings import TGA, DATA_BASE
import json
import time
import requests
import os

logger = logging.getLogger('download')


class DownloadUtil:
    def __init__(self, obj):
        self.base_server_url = TGA['base_url']
        self.token = TGA['token']
        self.format = 'csv'
        self.name = obj['name']
        self.sql = obj['sql']

    def run(self):
        try:
            preset_data = self._prepare()
            logger.info("download %s" % preset_data.text)
            ret = json.loads(preset_data.text)
            if 'return_code' in ret and ret['return_code'] == 0:
                return self._get_all(ret['data'])
        except Exception as e:
            logger.info('download %s' % e)
        return None

    def _get_all(self, preset_data):
        """ 获取所有数据
        获取分页信息，循环调用_get_data方法获取当前页面数据，并写入csv
        """
        task_id = preset_data['taskId']
        file_name = (self.name + '_' + time.strftime("%Y_%m_%d_%H_%M_%S", time.localtime()) + '.csv')
        path = os.path.join(DATA_BASE, 'tga_user_tag_csv_files', file_name)

        with open(path, 'w') as f:
            for i in range(preset_data['pageCount']):
                page_result = self._get_data(task_id, i)
                if page_result.text[0] == '{':
                    logger.error('download page %s', page_result.text)
                    return None
                f.write(page_result.text)
        rowCount = preset_data['rowCount']

        return {'filepath':path, 'rowCount':rowCount}

    def _prepare(self):
        """ 预处理函数

        预处理，返回分页信息
        """
        return requests.post(f'{self.base_server_url}/open/execute-sql', headers={
            'Content-Type': 'application/x-www-form-urlencoded',
        }, params={
            ('token', self.token),
        }, data={
            'format': self.format, 'sql': self.sql
        }, timeout=900)

    def _get_data(self, task_id, page_id):
        """ 获取单页数据

        根据page_id获取数据
        """
        return requests.get(f'{self.base_server_url}/open/sql-result-page', headers={
            'Content-Type': 'application/x-www-form-urlencoded',
        }, params=(
            ('token', self.token),
            ('taskId', task_id),
            ('pageId', page_id),
        ), timeout=900)


if __name__ == '__main__':
    du = DownloadUtil({
        'name': 'test',
        'sql': '''
          SELECT "$part_event" , count(distinct "#country_code") as country_code 
            from  ta.v_event_46
            WHERE    "$part_date" BETWEEN '2021-09-27' AND '2021-10-27' group by  "$part_event" order by count(distinct "#country_code")
           '''

    })
    result = du.run()
    print(result)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="tga tgautil",
    version="0.0.9",
    author="DSC Developer",
    author_email="1106896377@qq.com",
    description="Some useful util tools for tga",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jewdore/tga-utils.git",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)


import json
import logging
import time

from config.settings import TGA
from proj.models import CeleryTask
from tgautil.dingtalk import send_dingtalk_message

logger = logging.getLogger('raw_logger')

task_logger = logging.getLogger('task')


class BaseTask:
    def __init__(self, flow_id, params, depends=[]):
        # 接收参数
        self.flow_id = flow_id
        self.depends = depends
        self.task_result_dict = {
            'time_cost': 0,
        }

        self.task, create = CeleryTask.objects.get_or_create(flow_id=flow_id, task_name=self.task_name)

        if create:
            self.params = params
            self.task.params = params
            self.task.save(update_fields=['params', 'modified'])
        else:
            self.params = self.task.params
            self.task.retry_time = self.task.retry_time + 1
            self.task.save(update_fields=['retry_time', 'modified'])

        if 'depends' in self.params:
            self.depends = self.params['depends']
            
        self.run_status = 0  # 0 不可执行 1 可执行
        # 读取配置
        self.settings = TGA

    def handle(self):
        start_time = time.time()
        logger.info('handle task: flowid %s, task_name: %s, status %s', self.flow_id, self.task.task_name,
                    self.task.status)

        # injudge task should run or not
        if self.task.status == 2:
            return True
        elif self.task.status == 1:
            logger.info('celry task %s %s is running', self.flow_id, self.task.task_name)
            return False
        else:
            depend_success_count = CeleryTask.objects.filter(flow_id=self.flow_id, task_name__in=self.depends,
                                                             status=2).count()
            if depend_success_count < len(self.depends):
                logger.info('celry task %s %s prerequisite task not done', self.flow_id, self.task.task_name)
                return False

        # run task
        self.task.status = 1
        self.task.save(update_fields=['status', 'modified'])
        # process 由子类继承，必须返回true or false
        success, result = self.process()

        end_time = time.time()
        result['time_cost'] = round(end_time - start_time, 4)
        if success:
            self.task.status = 2
            self.task.result = result
            self.task.save(update_fields=['status', 'result', 'modified'])
        else:
            send_dingtalk_message(json.dumps({'result': result, 'flow_id': self.flow_id, 'params': self.params}))
            self.task.status = 3
            self.task.result = result
            self.task.save(update_fields=['status', 'result', 'modified'])

        task_logger.info(
            json.dumps(
                {
                    'cerated': self.task.created,
                    'task_name': self.task.task_name,
                    'status': self.task.status,
                    'params': self.task.params,
                    'retry_time': self.task.retry_time,
                    'result': self.task.result,
                }, indent=4, sort_keys=True, default=str
            )
        )

    def process(self):
        pass



from tgaflow.base import BaseTask
from tgautil.download_util import DownloadUtil
from tgautil.sql_format import SQLFormat


class DownloadTask(BaseTask):
    def __init__(self, flow_id, params):
        self.task_name = 'download'
        super().__init__(flow_id, params, ['check', 'refresh'])

    def process(self):
        sqlFormat = SQLFormat()
        du = DownloadUtil({
            'name': self.params['name'],
            'sql': sqlFormat.run(self.params)

        })
        result = None
        try_num = 0

        while (not result) and try_num <= 3:
            try:
                result = du.run()
            except Exception as e:
                try_num = try_num + 1
                if try_num <= 3:
                    logger.info( ' DownloadTask flow_id: %s . exception: %s . try num: %s ', self.flow_id, e, try_num)
                    time.sleep(3)
                else:
                    logger.error(' DownloadTask flow_id: %s . error: %s . try num: %s ', self.flow_id, e, try_num - 1)
                    self.send_ding_talk.send_dingtalk_message(
                        f' DownloadTask flow_id: {self.flow_id} . error: {e} . try num: {try_num - 1} ')
                    return

        self.task_result_dict['filepath'] = result['filepath']
        self.task_result_dict['download_count'] = result['rowCount']
        if result is None:
            return False, {'result':result}
        else:
            return True, self.task_result_dict



import re
from datetime import datetime, timedelta


class SQLFormat:
    def __init__(self):
        self.tga_template = '''select %s from ta.user_result_cluster_%s as t1 LEFT JOIN ta.v_user_%s as t2 ON t1."#user_id" = t2."#user_id" where cluster_name='%s' and "#account_id" is not null'''
        self.sql_place = {
            # 当天
            'cur_date': datetime.now().strftime('%Y-%m-%d'),
            # 昨天
            'last_date': (datetime.now() + timedelta(hours=-24)).strftime('%Y-%m-%d'),
            # 当前时间
            'cur_time': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
            # 上一小时的时间
            'last_hour': (datetime.now() + timedelta(hours=-1)).strftime('%Y-%m-%d %H:%M:%S'),
            # 七天前的日期
            'seven_day_before': (datetime.now() + timedelta(days=-7)).strftime('%Y-%m-%d'),
            # 七天前的时间
            'seven_day_before_time': (datetime.now() + timedelta(days=-7)).strftime('%Y-%m-%d %H:%M:%S'),
            # 一天前
            'one_day_before': (datetime.now() + timedelta(days=-1)).strftime('%Y-%m-%d'),
            # 一天前的时间
            'one_day_before_time': (datetime.now() + timedelta(days=-1)).strftime('%Y-%m-%d %H:%M:%S'),
            # 28小时前的日期
            'twenty_eight_hours_before_date': (datetime.now() + timedelta(hours=-28)).strftime('%Y-%m-%d'),
            # 28小时前的时间
            'twenty_eight_hours_before_time': (datetime.now() + timedelta(hours=-28)).strftime('%Y-%m-%d %H:%M:%S'),
        }

    # sql params[sql], tga params[fields, game_id, cluster_name]
    def run(self, params):
        if 'sql' in params:
            download_sql = self.sql_format(params['sql'])
        else:
            download_sql = self.tga_format(fields=params['fields'], game_id=params['game_id'],
                                           cluster_name=params['cluster_name'])
        return download_sql

    def sql_format(self, sql):
        placeholder_pattern = re.compile(r'\$\{\w+\}')
        format_arr = placeholder_pattern.findall(sql)

        for placeholder in format_arr:
            format_type = placeholder.split('$', 1)[1].strip('{}')
            if format_type in self.sql_place:
                sql = sql.replace(placeholder, self.sql_place.get(format_type))
        return sql

    def tga_format(self, fields, game_id, cluster_name):
        sql = self.tga_template % (fields, game_id, game_id, cluster_name)
        return sql
